@@ -2,7 +2,7 @@ package Archive::Zip;
 
 use strict;
 BEGIN {
-	require 5.003_96;
+    require 5.003_96;
 }
 use UNIVERSAL           ();
 use Carp                ();
@@ -15,132 +15,132 @@ use FileHandle          ();
 
 use vars qw( $VERSION @ISA );
 BEGIN {
-	$VERSION = '1.28';
+    $VERSION = '1.29';
 
-	require Exporter;
-	@ISA = qw( Exporter );
+    require Exporter;
+    @ISA = qw( Exporter );
 }
 
 use vars qw( $ChunkSize $ErrorHandler );
 BEGIN {
-	# This is the size we'll try to read, write, and (de)compress.
-	# You could set it to something different if you had lots of memory
-	# and needed more speed.
-	$ChunkSize ||= 32768;
+    # This is the size we'll try to read, write, and (de)compress.
+    # You could set it to something different if you had lots of memory
+    # and needed more speed.
+    $ChunkSize ||= 32768;
 
-	$ErrorHandler = \&Carp::carp;
+    $ErrorHandler = \&Carp::carp;
 }
 
 # BEGIN block is necessary here so that other modules can use the constants.
 use vars qw( @EXPORT_OK %EXPORT_TAGS );
 BEGIN {
-	@EXPORT_OK   = ('computeCRC32');
-	%EXPORT_TAGS = (
-		CONSTANTS => [ qw(
-			FA_MSDOS
-			FA_UNIX
-			GPBF_ENCRYPTED_MASK
-			GPBF_DEFLATING_COMPRESSION_MASK
-			GPBF_HAS_DATA_DESCRIPTOR_MASK
-			COMPRESSION_STORED
-			COMPRESSION_DEFLATED
-			COMPRESSION_LEVEL_NONE
-			COMPRESSION_LEVEL_DEFAULT
-			COMPRESSION_LEVEL_FASTEST
-			COMPRESSION_LEVEL_BEST_COMPRESSION
-			IFA_TEXT_FILE_MASK
-			IFA_TEXT_FILE
-			IFA_BINARY_FILE
-			) ],
+    @EXPORT_OK   = ('computeCRC32');
+    %EXPORT_TAGS = (
+        CONSTANTS => [ qw(
+            FA_MSDOS
+            FA_UNIX
+            GPBF_ENCRYPTED_MASK
+            GPBF_DEFLATING_COMPRESSION_MASK
+            GPBF_HAS_DATA_DESCRIPTOR_MASK
+            COMPRESSION_STORED
+            COMPRESSION_DEFLATED
+            COMPRESSION_LEVEL_NONE
+            COMPRESSION_LEVEL_DEFAULT
+            COMPRESSION_LEVEL_FASTEST
+            COMPRESSION_LEVEL_BEST_COMPRESSION
+            IFA_TEXT_FILE_MASK
+            IFA_TEXT_FILE
+            IFA_BINARY_FILE
+            ) ],
 
-		MISC_CONSTANTS => [ qw(
-			FA_AMIGA
-			FA_VAX_VMS
-			FA_VM_CMS
-			FA_ATARI_ST
-			FA_OS2_HPFS
-			FA_MACINTOSH
-			FA_Z_SYSTEM
-			FA_CPM
-			FA_TOPS20
-			FA_WINDOWS_NTFS
-			FA_QDOS
-			FA_ACORN
-			FA_VFAT
-			FA_MVS
-			FA_BEOS
-			FA_TANDEM
-			FA_THEOS
-			GPBF_IMPLODING_8K_SLIDING_DICTIONARY_MASK
-			GPBF_IMPLODING_3_SHANNON_FANO_TREES_MASK
-			GPBF_IS_COMPRESSED_PATCHED_DATA_MASK
-			COMPRESSION_SHRUNK
-			DEFLATING_COMPRESSION_NORMAL
-			DEFLATING_COMPRESSION_MAXIMUM
-			DEFLATING_COMPRESSION_FAST
-			DEFLATING_COMPRESSION_SUPER_FAST
-			COMPRESSION_REDUCED_1
-			COMPRESSION_REDUCED_2
-			COMPRESSION_REDUCED_3
-			COMPRESSION_REDUCED_4
-			COMPRESSION_IMPLODED
-			COMPRESSION_TOKENIZED
-			COMPRESSION_DEFLATED_ENHANCED
-			COMPRESSION_PKWARE_DATA_COMPRESSION_LIBRARY_IMPLODED
-			) ],
+        MISC_CONSTANTS => [ qw(
+            FA_AMIGA
+            FA_VAX_VMS
+            FA_VM_CMS
+            FA_ATARI_ST
+            FA_OS2_HPFS
+            FA_MACINTOSH
+            FA_Z_SYSTEM
+            FA_CPM
+            FA_TOPS20
+            FA_WINDOWS_NTFS
+            FA_QDOS
+            FA_ACORN
+            FA_VFAT
+            FA_MVS
+            FA_BEOS
+            FA_TANDEM
+            FA_THEOS
+            GPBF_IMPLODING_8K_SLIDING_DICTIONARY_MASK
+            GPBF_IMPLODING_3_SHANNON_FANO_TREES_MASK
+            GPBF_IS_COMPRESSED_PATCHED_DATA_MASK
+            COMPRESSION_SHRUNK
+            DEFLATING_COMPRESSION_NORMAL
+            DEFLATING_COMPRESSION_MAXIMUM
+            DEFLATING_COMPRESSION_FAST
+            DEFLATING_COMPRESSION_SUPER_FAST
+            COMPRESSION_REDUCED_1
+            COMPRESSION_REDUCED_2
+            COMPRESSION_REDUCED_3
+            COMPRESSION_REDUCED_4
+            COMPRESSION_IMPLODED
+            COMPRESSION_TOKENIZED
+            COMPRESSION_DEFLATED_ENHANCED
+            COMPRESSION_PKWARE_DATA_COMPRESSION_LIBRARY_IMPLODED
+            ) ],
 
-		ERROR_CODES => [ qw(
-			AZ_OK
-			AZ_STREAM_END
-			AZ_ERROR
-			AZ_FORMAT_ERROR
-			AZ_IO_ERROR
-			) ],
+        ERROR_CODES => [ qw(
+            AZ_OK
+            AZ_STREAM_END
+            AZ_ERROR
+            AZ_FORMAT_ERROR
+            AZ_IO_ERROR
+            ) ],
 
-		# For Internal Use Only
-		PKZIP_CONSTANTS => [ qw(
-			SIGNATURE_FORMAT
-			SIGNATURE_LENGTH
-			LOCAL_FILE_HEADER_SIGNATURE
-			LOCAL_FILE_HEADER_FORMAT
-			LOCAL_FILE_HEADER_LENGTH
-			CENTRAL_DIRECTORY_FILE_HEADER_SIGNATURE
-			DATA_DESCRIPTOR_FORMAT
-			DATA_DESCRIPTOR_LENGTH
-			DATA_DESCRIPTOR_SIGNATURE
-			DATA_DESCRIPTOR_FORMAT_NO_SIG
-			DATA_DESCRIPTOR_LENGTH_NO_SIG
-			CENTRAL_DIRECTORY_FILE_HEADER_FORMAT
-			CENTRAL_DIRECTORY_FILE_HEADER_LENGTH
-			END_OF_CENTRAL_DIRECTORY_SIGNATURE
-			END_OF_CENTRAL_DIRECTORY_SIGNATURE_STRING
-			END_OF_CENTRAL_DIRECTORY_FORMAT
-			END_OF_CENTRAL_DIRECTORY_LENGTH
-			) ],
+        # For Internal Use Only
+        PKZIP_CONSTANTS => [ qw(
+            SIGNATURE_FORMAT
+            SIGNATURE_LENGTH
+            LOCAL_FILE_HEADER_SIGNATURE
+            LOCAL_FILE_HEADER_FORMAT
+            LOCAL_FILE_HEADER_LENGTH
+            CENTRAL_DIRECTORY_FILE_HEADER_SIGNATURE
+            DATA_DESCRIPTOR_FORMAT
+            DATA_DESCRIPTOR_LENGTH
+            DATA_DESCRIPTOR_SIGNATURE
+            DATA_DESCRIPTOR_FORMAT_NO_SIG
+            DATA_DESCRIPTOR_LENGTH_NO_SIG
+            CENTRAL_DIRECTORY_FILE_HEADER_FORMAT
+            CENTRAL_DIRECTORY_FILE_HEADER_LENGTH
+            END_OF_CENTRAL_DIRECTORY_SIGNATURE
+            END_OF_CENTRAL_DIRECTORY_SIGNATURE_STRING
+            END_OF_CENTRAL_DIRECTORY_FORMAT
+            END_OF_CENTRAL_DIRECTORY_LENGTH
+            ) ],
 
-		# For Internal Use Only
-		UTILITY_METHODS => [ qw(
-			_error
-			_printError
-			_ioError
-			_formatError
-			_subclassResponsibility
-			_binmode
-			_isSeekable
-			_newFileHandle
-			_readSignature
-			_asZipDirName
-			) ],
-	);
+        # For Internal Use Only
+        UTILITY_METHODS => [ qw(
+            _error
+            _printError
+            _ioError
+            _formatError
+            _subclassResponsibility
+            _binmode
+            _isSeekable
+            _newFileHandle
+            _readSignature
+            _asZipDirName
+            ) ],
+    );
 
-	# Add all the constant names and error code names to @EXPORT_OK
-	Exporter::export_ok_tags( qw(
-		CONSTANTS
-		ERROR_CODES
-		PKZIP_CONSTANTS
-		UTILITY_METHODS
-		MISC_CONSTANTS
-		) );
+    # Add all the constant names and error code names to @EXPORT_OK
+    Exporter::export_ok_tags( qw(
+        CONSTANTS
+        ERROR_CODES
+        PKZIP_CONSTANTS
+        UTILITY_METHODS
+        MISC_CONSTANTS
+        ) );
 
 }
 
@@ -230,7 +230,7 @@ use constant CENTRAL_DIRECTORY_FILE_HEADER_LENGTH     => 42;
 
 use constant END_OF_CENTRAL_DIRECTORY_SIGNATURE        => 0x06054b50;
 use constant END_OF_CENTRAL_DIRECTORY_SIGNATURE_STRING =>
-	pack( "V", END_OF_CENTRAL_DIRECTORY_SIGNATURE );
+    pack( "V", END_OF_CENTRAL_DIRECTORY_SIGNATURE );
 use constant END_OF_CENTRAL_DIRECTORY_FORMAT           => "v4 V2 v";
 use constant END_OF_CENTRAL_DIRECTORY_LENGTH           => 18;
 
@@ -264,14 +264,14 @@ use constant ZIPMEMBERCLASS  => 'Archive::Zip::Member';
 # Convenience functions
 
 sub _ISA ($$) {
-	# Can't rely on Scalar::Util, so use the next best way
-	local $@;
-	!! eval { ref $_[0] and $_[0]->isa($_[1]) };
+    # Can't rely on Scalar::Util, so use the next best way
+    local $@;
+    !! eval { ref $_[0] and $_[0]->isa($_[1]) };
 }
 
 sub _CAN ($$) {
-	local $@;
-	!! eval { ref $_[0] and $_[0]->can($_[1]) };
+    local $@;
+    !! eval { ref $_[0] and $_[0]->can($_[1]) };
 }
 
 
@@ -282,37 +282,46 @@ sub _CAN ($$) {
 # Methods
 
 sub new {
-	my $class = shift;
-	return $class->ZIPARCHIVECLASS->new(@_);
+    my $class = shift;
+    return $class->ZIPARCHIVECLASS->new(@_);
 }
 
 sub computeCRC32 {
-	my $data = shift;
-	$data = shift if ref($data);    # allow calling as an obj method
-	my $crc = shift;
+    my ( $data, $crc );
+
+    if ( ref( $_[0] ) eq 'HASH' ) {
+        $data = $_[0]->{string};
+        $crc  = $_[0]->{checksum};
+    }
+    else {
+        $data = shift;
+        $data = shift if ref($data);
+        $crc  = shift;
+    }
+
 	return Compress::Raw::Zlib::crc32( $data, $crc );
 }
 
 # Report or change chunk size used for reading and writing.
 # Also sets Zlib's default buffer size (eventually).
 sub setChunkSize {
-	my $chunkSize = shift;
-	$chunkSize = shift if ref($chunkSize);    # object method on zip?
-	my $oldChunkSize = $Archive::Zip::ChunkSize;
-	$Archive::Zip::ChunkSize = $chunkSize if ($chunkSize);
-	return $oldChunkSize;
+    shift if ref( $_[0] ) eq 'Archive::Zip::Archive';
+    my $chunkSize = ( ref( $_[0] ) eq 'HASH' ) ? shift->{chunkSize} : shift;
+    my $oldChunkSize = $Archive::Zip::ChunkSize;
+    $Archive::Zip::ChunkSize = $chunkSize if ($chunkSize);
+    return $oldChunkSize;
 }
 
 sub chunkSize {
-	return $Archive::Zip::ChunkSize;
+    return $Archive::Zip::ChunkSize;
 }
 
-sub setErrorHandler (&) {
-	my $errorHandler = shift;
-	$errorHandler = \&Carp::carp unless defined($errorHandler);
-	my $oldErrorHandler = $Archive::Zip::ErrorHandler;
-	$Archive::Zip::ErrorHandler = $errorHandler;
-	return $oldErrorHandler;
+sub setErrorHandler {
+    my $errorHandler = ( ref( $_[0] ) eq 'HASH' ) ? shift->{subroutine} : shift;
+    $errorHandler = \&Carp::carp unless defined($errorHandler);
+    my $oldErrorHandler = $Archive::Zip::ErrorHandler;
+    $Archive::Zip::ErrorHandler = $errorHandler;
+    return $oldErrorHandler;
 }
 
 
@@ -323,74 +332,74 @@ sub setErrorHandler (&) {
 # Private utility functions (not methods).
 
 sub _printError {
-	my $string = join ( ' ', @_, "\n" );
-	my $oldCarpLevel = $Carp::CarpLevel;
-	$Carp::CarpLevel += 2;
-	&{$ErrorHandler} ($string);
-	$Carp::CarpLevel = $oldCarpLevel;
+    my $string = join ( ' ', @_, "\n" );
+    my $oldCarpLevel = $Carp::CarpLevel;
+    $Carp::CarpLevel += 2;
+    &{$ErrorHandler} ($string);
+    $Carp::CarpLevel = $oldCarpLevel;
 }
 
 # This is called on format errors.
 sub _formatError {
-	shift if ref( $_[0] );
-	_printError( 'format error:', @_ );
-	return AZ_FORMAT_ERROR;
+    shift if ref( $_[0] );
+    _printError( 'format error:', @_ );
+    return AZ_FORMAT_ERROR;
 }
 
 # This is called on IO errors.
 sub _ioError {
-	shift if ref( $_[0] );
-	_printError( 'IO error:', @_, ':', $! );
-	return AZ_IO_ERROR;
+    shift if ref( $_[0] );
+    _printError( 'IO error:', @_, ':', $! );
+    return AZ_IO_ERROR;
 }
 
 # This is called on generic errors.
 sub _error {
-	shift if ref( $_[0] );
-	_printError( 'error:', @_ );
-	return AZ_ERROR;
+    shift if ref( $_[0] );
+    _printError( 'error:', @_ );
+    return AZ_ERROR;
 }
 
 # Called when a subclass should have implemented
 # something but didn't
 sub _subclassResponsibility {
-	Carp::croak("subclass Responsibility\n");
+    Carp::croak("subclass Responsibility\n");
 }
 
 # Try to set the given file handle or object into binary mode.
 sub _binmode {
-	my $fh = shift;
-	return _CAN( $fh, 'binmode' ) ? $fh->binmode() : binmode($fh);
+    my $fh = shift;
+    return _CAN( $fh, 'binmode' ) ? $fh->binmode() : binmode($fh);
 }
 
 # Attempt to guess whether file handle is seekable.
 # Because of problems with Windows, this only returns true when
 # the file handle is a real file.  
 sub _isSeekable {
-	my $fh = shift;
-	return 0 unless ref $fh;
-	if ( _ISA($fh, 'IO::Scalar') ) {
-		# IO::Scalar objects are brokenly-seekable
-		return 0;
-	}
-	if ( _ISA($fh, 'IO::String') ) {
-		return 1;
-	}
-	if ( _ISA($fh, 'IO::Seekable') ) {
-		# Unfortunately, some things like FileHandle objects
-		# return true for Seekable, but AREN'T!!!!!
-		if ( _ISA($fh, 'FileHandle') ) {
-			return 0;
-		} else {
-			return 1;
-		}
-	}
-	if ( _CAN($fh, 'stat') ) {
-		return -f $fh;
-	}
-	return (
-		_CAN($fh, 'seek') and _CAN($fh, 'tell')
-		) ? 1 : 0;
+    my $fh = shift;
+    return 0 unless ref $fh;
+    if ( _ISA($fh, 'IO::Scalar') ) {
+        # IO::Scalar objects are brokenly-seekable
+        return 0;
+    }
+    if ( _ISA($fh, 'IO::String') ) {
+        return 1;
+    }
+    if ( _ISA($fh, 'IO::Seekable') ) {
+        # Unfortunately, some things like FileHandle objects
+        # return true for Seekable, but AREN'T!!!!!
+        if ( _ISA($fh, 'FileHandle') ) {
+            return 0;
+        } else {
+            return 1;
+        }
+    }
+    if ( _CAN($fh, 'stat') ) {
+        return -f $fh;
+    }
+    return (
+        _CAN($fh, 'seek') and _CAN($fh, 'tell')
+        ) ? 1 : 0;
 }
 
 # Print to the filehandle, while making sure the pesky Perl special global 
@@ -410,25 +419,25 @@ sub _print
 # Or, if given something that is a ref but not an IO::Handle,
 # passes back the same thing.
 sub _newFileHandle {
-	my $fd     = shift;
-	my $status = 1;
-	my $handle;
+    my $fd     = shift;
+    my $status = 1;
+    my $handle;
 
-	if ( ref($fd) ) {
-		if ( _ISA($fd, 'IO::Scalar') or _ISA($fd, 'IO::String') ) {
-			$handle = $fd;
-		} elsif ( _ISA($fd, 'IO::Handle') or ref($fd) eq 'GLOB' ) {
-			$handle = IO::File->new;
-			$status = $handle->fdopen( $fd, @_ );
-		} else {
-			$handle = $fd;
-		}
-	} else {
-		$handle = IO::File->new;
-		$status = $handle->open( $fd, @_ );
-	}
+    if ( ref($fd) ) {
+        if ( _ISA($fd, 'IO::Scalar') or _ISA($fd, 'IO::String') ) {
+            $handle = $fd;
+        } elsif ( _ISA($fd, 'IO::Handle') or ref($fd) eq 'GLOB' ) {
+            $handle = IO::File->new;
+            $status = $handle->fdopen( $fd, @_ );
+        } else {
+            $handle = $fd;
+        }
+    } else {
+        $handle = IO::File->new;
+        $status = $handle->open( $fd, @_ );
+    }
 
-	return ( $status, $handle );
+    return ( $status, $handle );
 }
 
 # Returns next signature from given file handle, leaves
@@ -437,37 +446,37 @@ sub _newFileHandle {
 # ( $status, $signature) = _readSignature( $fh, $fileName );
 
 sub _readSignature {
-	my $fh                = shift;
-	my $fileName          = shift;
-	my $expectedSignature = shift;    # optional
+    my $fh                = shift;
+    my $fileName          = shift;
+    my $expectedSignature = shift;    # optional
 
-	my $signatureData;
-	my $bytesRead = $fh->read( $signatureData, SIGNATURE_LENGTH );
-	if ( $bytesRead != SIGNATURE_LENGTH ) {
-		return _ioError("reading header signature");
-	}
-	my $signature = unpack( SIGNATURE_FORMAT, $signatureData );
-	my $status    = AZ_OK;
+    my $signatureData;
+    my $bytesRead = $fh->read( $signatureData, SIGNATURE_LENGTH );
+    if ( $bytesRead != SIGNATURE_LENGTH ) {
+        return _ioError("reading header signature");
+    }
+    my $signature = unpack( SIGNATURE_FORMAT, $signatureData );
+    my $status    = AZ_OK;
 
-	# compare with expected signature, if any, or any known signature.
-	if ( ( defined($expectedSignature) && $signature != $expectedSignature )
-		|| ( !defined($expectedSignature)
-			&& $signature != CENTRAL_DIRECTORY_FILE_HEADER_SIGNATURE
-			&& $signature != LOCAL_FILE_HEADER_SIGNATURE
-			&& $signature != END_OF_CENTRAL_DIRECTORY_SIGNATURE
-			&& $signature != DATA_DESCRIPTOR_SIGNATURE ) )
-	{
-		my $errmsg = sprintf( "bad signature: 0x%08x", $signature );
-		if ( _isSeekable($fh) )
-		{
-			$errmsg .=
-			  sprintf( " at offset %d", $fh->tell() - SIGNATURE_LENGTH );
-		}
+    # compare with expected signature, if any, or any known signature.
+    if ( ( defined($expectedSignature) && $signature != $expectedSignature )
+        || ( !defined($expectedSignature)
+            && $signature != CENTRAL_DIRECTORY_FILE_HEADER_SIGNATURE
+            && $signature != LOCAL_FILE_HEADER_SIGNATURE
+            && $signature != END_OF_CENTRAL_DIRECTORY_SIGNATURE
+            && $signature != DATA_DESCRIPTOR_SIGNATURE ) )
+    {
+        my $errmsg = sprintf( "bad signature: 0x%08x", $signature );
+        if ( _isSeekable($fh) )
+        {
+            $errmsg .=
+              sprintf( " at offset %d", $fh->tell() - SIGNATURE_LENGTH );
+        }
 
-		$status = _formatError("$errmsg in file $fileName");
-	}
+        $status = _formatError("$errmsg in file $fileName");
+    }
 
-	return ( $status, $signature );
+    return ( $status, $signature );
 }
 
 # Utility method to make and open a temp file.
@@ -479,15 +488,15 @@ sub _readSignature {
 #
 
 sub tempFile {
-	my $dir = shift;
-	my ( $fh, $filename ) = File::Temp::tempfile(
-		SUFFIX => '.zip',
-		UNLINK => 0,        # we will delete it!
-		$dir ? ( DIR => $dir ) : ()
-	);
-	return ( undef, undef ) unless $fh;
-	my ( $status, $newfh ) = _newFileHandle( $fh, 'w+' );
-	return ( $newfh, $filename );
+    my $dir = ( ref( $_[0] ) eq 'HASH' ) ? shift->{tempDir} : shift;
+    my ( $fh, $filename ) = File::Temp::tempfile(
+        SUFFIX => '.zip',
+        UNLINK => 0,        # we will delete it!
+        $dir ? ( DIR => $dir ) : ()
+    );
+    return ( undef, undef ) unless $fh;
+    my ( $status, $newfh ) = _newFileHandle( $fh, 'w+' );
+    return ( $newfh, $filename );
 }
 
 # Return the normalized directory name as used in a zip file (path
@@ -500,26 +509,26 @@ sub tempFile {
 # If third argument is a reference, returns volume information there.
 #
 # input         output
-# .				('.')	'.'
-# ./a			('a')	a
-# ./a/b			('a','b')	a/b
-# ./a/b/		('a','b')	a/b
-# a/b/			('a','b')	a/b
-# /a/b/			('','a','b')	/a/b
-# c:\a\b\c.doc	('','a','b','c.doc')	/a/b/c.doc		# on Windoze
-# "i/o maps:whatever"	('i_o maps', 'whatever')  "i_o maps/whatever"	# on Macs
+# .             ('.')   '.'
+# ./a           ('a')   a
+# ./a/b         ('a','b')   a/b
+# ./a/b/        ('a','b')   a/b
+# a/b/          ('a','b')   a/b
+# /a/b/         ('','a','b')    /a/b
+# c:\a\b\c.doc  ('','a','b','c.doc')    /a/b/c.doc      # on Windoze
+# "i/o maps:whatever"   ('i_o maps', 'whatever')  "i_o maps/whatever"   # on Macs
 sub _asZipDirName    
 {
-	my $name      = shift;
-	my $forceDir  = shift;
-	my $volReturn = shift;
-	my ( $volume, $directories, $file ) =
-	  File::Spec->splitpath( File::Spec->canonpath($name), $forceDir );
-	$$volReturn = $volume if ( ref($volReturn) );
-	my @dirs = map { $_ =~ s{/}{_}g; $_ } File::Spec->splitdir($directories);
+    my $name      = shift;
+    my $forceDir  = shift;
+    my $volReturn = shift;
+    my ( $volume, $directories, $file ) =
+      File::Spec->splitpath( File::Spec->canonpath($name), $forceDir );
+    $$volReturn = $volume if ( ref($volReturn) );
+    my @dirs = map { $_ =~ s{/}{_}g; $_ } File::Spec->splitdir($directories);
     if ( @dirs > 0 ) { pop (@dirs) unless $dirs[-1] }   # remove empty component
     push ( @dirs, defined($file) ? $file : '' );
-	#return wantarray ? @dirs : join ( '/', @dirs );
+    #return wantarray ? @dirs : join ( '/', @dirs );
     return join ( '/', @dirs );
 }
 
@@ -529,18 +538,20 @@ sub _asZipDirName
 #
 sub _asLocalName    
 {
-	my $name   = shift;    # zip format
-	my $volume = shift;
-	$volume = '' unless defined($volume);    # local FS format
+    my $name   = shift;    # zip format
+    my $volume = shift;
+    $volume = '' unless defined($volume);    # local FS format
 
-	my @paths = split ( /\//, $name );
-	my $filename = pop (@paths);
-	$filename = '' unless defined($filename);
-	my $localDirs = @paths?File::Spec->catdir(@paths):'';
-	my $localName = File::Spec->catpath( $volume, $localDirs, $filename );
-    use Cwd;
-    $localName = File::Spec->catfile(getcwd, $localName) unless $volume;
-	return $localName;
+    my @paths = split ( /\//, $name );
+    my $filename = pop (@paths);
+    $filename = '' unless defined($filename);
+    my $localDirs = @paths ? File::Spec->catdir(@paths) : '';
+    my $localName = File::Spec->catpath( $volume, $localDirs, $filename );
+    require Cwd;
+    unless ( $volume ) {
+        $localName = File::Spec->rel2abs( $localName, Cwd::getcwd );
+    }
+    return $localName;
 }
 
 1;
@@ -1267,7 +1278,7 @@ You can truncate the zip by seeking backwards and then writing the
 directory:
 
     my $fh = IO::File->new( 'someFile.zip', 'w' );
-    	my $retval = $zip->writeToFileHandle( $fh );
+        my $retval = $zip->writeToFileHandle( $fh );
     if ( $retval == AZ_IO_ERROR ) {
         my @unwritten = grep { not $_->wasWritten() } $zip->members();
         if (@unwritten) {
@@ -1829,7 +1840,7 @@ in chunks using these methods:
     {
     ( $bufferRef, $status ) = $member->readChunk();
     die "error $status"
-    			if $status != AZ_OK && $status != AZ_STREAM_END;
+                if $status != AZ_OK && $status != AZ_STREAM_END;
     # do something with $bufferRef:
     print $$bufferRef;
     }
